from enum import Enum


class Category(Enum):

    root = 371

    electricity = 0

    seds = 40203

    petroleum = 714755

    natural_gas = 714804

    total_energy = 711224

    coal = 717234

    steo = 829714

    aeo = 964164

    crude_oil = 1292190

    international_energy = 2134384

    usesod = 2123635

    co2_emissions = 2251604

    us_nuclear_outages = 2889994
